﻿/// <reference name="MicrosoftAjax.js"/>

Type.registerNamespace("Bleroy.Sample");

Bleroy.Sample.EditInPlace = function(element) {
    /// <summary>Transforms a text input into an edit in place control.</summary>
    /// <param name="element" domElement="true">The input to extend.</param>
    var tagName = element.tagName.toUpperCase();
    if (tagName !== "INPUT" && tagName !== "TEXTAREA") {
        throw Error.argument("element", "Element must be an input or a textarea.");
    }
    Bleroy.Sample.EditInPlace.initializeBase(this, [element]);
    this._span = this._oldVisibility = this._class = this._inputBlur = null;
}
Bleroy.Sample.EditInPlace.prototype = {
    get_cssClass: function() {
        /// <value type="String">The class to apply to the label.</value>
        return this._class;
    },
    set_cssClass: function(value) {
        if (this._span) {
            Sys.UI.DomElement.removeCssClass(this._span, this._class);
            Sys.UI.DomElement.addCssClass(this._span, value);
        }
        this._class = value;
    },
    get_isEditing: function() {
        /// <value type="Boolean">True if in edit mode.</value>
        return this.get_element().style.visibility !== "hidden";
    },
    beginEdit: function() {
        /// <summary>Puts the behavior in edit mode</summary>
        var elt = this.get_element();
        this._span.style.visibility = "hidden";
        elt.style.visibility = this._oldVisibility;
        elt.focus();
        this.raisePropertyChanged("isEditing");
    },
    endEdit: function() {
        /// <summary>Puts the behavior out of edit mode</summary>
        var elt = this.get_element();
        this._span.innerHTML = elt.value;
        this._span.style.visibility = this._oldVisibility;
        elt.style.visibility = "hidden";
        this.raisePropertyChanged("isEditing");
    },
    initialize: function() {
        Bleroy.Sample.EditInPlace.callBaseMethod(this, "initialize");
        var elt = this.get_element(),
            span = this._span = document.createElement("SPAN");
        var bounds = Sys.UI.DomElement.getBounds(elt);
        span.style.position = "absolute";
        Sys.UI.DomElement.setLocation(span, bounds.x, bounds.y);
        span.style.width = bounds.width + "px";
        span.style.height = bounds.height + "px";
        span.tabIndex = elt.tabIndex;
        span.className = this._class;
        Sys.UI.DomEvent.addHandlers(this._span, {
            click: this.beginEdit,
            focus: this.beginEdit
        }, this);
        elt.parentNode.insertBefore(this._span, elt);
        this._oldVisibility = elt.style.visibility;
        this._inputBlur = Function.createDelegate(this, this.endEdit);
        Sys.UI.DomEvent.addHandler(elt, "blur", this._inputBlur);
        this.endEdit();
    },
    dispose: function() {
        if (this._span) {
            Sys.UI.DomEvent.clearHandlers(this._span);
            this._span.parentNode.removeChild(this._span);
            var elt = this.get_element();
            Sys.UI.DomEvent.removeHandler(elt, "blur", this._inputBlur);
            elt.style.visibility = this._oldVisibility;
            this._inputBlur = this._spanClick = this._span = null;
        }
        Bleroy.Sample.EditInPlace.callBaseMethod(this, "dispose");
    }
}
Bleroy.Sample.EditInPlace.registerClass("Bleroy.Sample.EditInPlace", Sys.UI.Behavior);